# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import NMesh, Image, Object
import BZcommon
from BZcommon import retList,rnd
from BZobject import bzmesh,BlObject
import sys
import BZtypeList


class importer(bzmesh):

	def __init__(self,bzobj):
		self.obj=bzobj
		self.childlist=[]
		self.childlist.append(None)
		self.matLib={}
		self.matLib['BZTexface']=0
		self.matIndex=0

	def run(self):
		self.blobj=NMesh.New(self.obj.name)
		self.blobj.addMaterial(Blender.Material.Get('BZtexface'))
		if self.obj.matref:
			self.blobj.addMaterial(Blender.Material.Get(self.obj.matref))
			self.matIndex+=1
			self.matLib[self.obj.matref]=self.matIndex
		self.blobj.hasFaceUV(1)
		for verts in self.obj.vertex:
			x,y,z=float(verts[0]),float(verts[1]),float(verts[2])
			v=NMesh.Vert(x,y,z)
			self.blobj.verts.append(v)
		for faces in self.obj.face:
			try:
				faces.vertices[4]
				print '\033[1;31mToo many vertices for a face:- \033[0m%s' % faces.vertices
			except IndexError:
				self.makeface(faces)
		try:#make inside points
			if self.obj.inside[0][0]:
				insidem=NMesh.New('%s_I' % self.obj.name)
				for verts in self.obj.inside:
					x,y,z=float(verts[0]),float(verts[1]),float(verts[2])
					v=NMesh.Vert(x,y,z)
					insidem.verts.append(v)
				inside=Object.New('Mesh','%s_I' % self.obj.name)
				inside.link(insidem)
				scene=Blender.Scene.GetCurrent ()
				scene.link (inside)
		except IndexError:
			inside=False
		try:#make outside points
			self.obj.outside[0]
			outsidem=NMesh.New('%s_O' % self.obj.name)
			for verts in self.obj.inside:
				x,y,z=float(verts[0]),float(verts[1]),float(verts[2])
				v=NMesh.Vert(x,y,z)
				outsidem.verts.append(v)
			outside=Object.New('Mesh','%s_O' % self.obj.name)
			outside.link(outsidem)
			scene=Blender.Scene.GetCurrent ()
			scene.link (outside)
		except IndexError:
			outside=False
		child=[]
		if outside:
			child.append(outside)
		if inside:
			child.append(inside)
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(self.blobj)
		scene = Blender.Scene.GetCurrent ()
		scene.link (scobj)
		scobj.makeParent(child)
		self.updateto()	
		Blender.Redraw()
	#enddef


	def makeface(self,face):
		if face.matref:
			#if type(face.matref)==type([]):face.matref=face.matref[0]
			image=self.getimage(face.matref)
			if not self.matLib.has_key(face.matref):
				try:
					self.blobj.materials[15]
					print '\033[1;31mToo many materials used on one mesh, try splitting it up\nExcess Material was %s. Importer may now Fail\033[0m' % face.matref
				except IndexError:
					self.blobj.addMaterial(Blender.Material.Get(face.matref))
					self.matIndex+=1
					self.matLib[face.matref]=self.matIndex
		else: 
			image=False
		f=NMesh.Face()
		#i=0
		'''vertex and face normals are read only in blender.'''
		for verts in face.vertices:
			f.v.append(self.blobj.verts[int(verts)])

		if face.matref:
			if image: f.image=image
			f.mat=self.matLib[face.matref]
			f.flag=1
		elif self.obj.matref:
			if self.getimage(self.obj.matref): f.image=self.getimage(self.obj.matref)
			f.mat=1
			f.flag=1
		f.mode=5
		tmode=0
		if face.shootthrough:
			tmode+=1
		if face.drivethrough:
			tmode+=2
		if face.passable:
			tmode+=4
		if face.smoothbounce:
			tmode+=8
		f.transp=tmode
		if face.texcoords:
			uv=[]
			for cord in face.texcoords:
				x=float(self.obj.texcoord[int(cord)][0])
				y=float(self.obj.texcoord[int(cord)][1])
				uv.append((x,y))
			f.uv=uv
		else:
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]
		self.blobj.faces.append(f)
	#enddef

	def getimage(self,matref):
		mat=Blender.Material.Get(matref)
		try:
			tex=mat.getTextures()[0].tex
			img=tex.getImage()
		except AttributeError:
			#print 'Texture not found %s' % matref
			img=None
		return img
	#enddef


class export(BlObject):
	def __init__(self,blobj,pipe):
		BZPrefs=Blender.Registry.GetKey('BZFlag', True)
		self.pipe=pipe
		self.out=BZtypeList.mesh()
		self.obj=blobj
		self.ro=BZPrefs['ro']
		self.coordIndex=[]
		self.normalIndex=[]
		self.vertIndex=[]
		self.matDict={}

	def run(self):
		self.updateFrom()
		self.makeMatDict()
		self.checkMatDict()
		self.pipe.write('mesh\n')
		self.pipe.write('\tname %s\n' % self.obj.name)
		self.writeBools()
		self.writeStrings()
		self.writeVertices()
		self.writeInside()
		self.writeOutside()
		if not (self.obj.getData().mode & Blender.NMesh.Modes.NOVNORMALSFLIP): 
			self.writeNormalList() ##exclude normals
		else:
			print 'Skipping normals for object %s.' % self.obj.name
		self.writeTexCoordList()
		self.writeFaces()
		self.writeLOD()
		self.writeMesh()
		self.pipe.write('end\n\n')
		self.pipe.flush()

	def writeTexCoordList(self):
		mesh=self.obj.getData()
		for face in mesh.faces:
			if face.uv:
				for coord in face.uv:
					data='%s %s' %(rnd(coord[0],self.ro),rnd(coord[1],self.ro))
					try:
						self.coordIndex.index(data)
					except ValueError:
						self.pipe.write("\ttexcoord %s\n" %data)
						self.coordIndex.append(data)
						self.pipe.flush()

	def writeNormalList(self):
		mesh=self.obj.getData()
		for vert in mesh.verts:
			v=vert.no
			data='%s %s %s' %(rnd(v[0],self.ro),rnd(v[1],self.ro),rnd(v[2],self.ro))
			try:
				self.normalIndex.index(data)
			except ValueError:
				self.pipe.write("\tnormal %s\n" %data)
				self.normalIndex.append(data)
				self.pipe.flush()

	def checkMatDict(self):
		mesh=self.obj.getData()
		#check the mesh face images have materials exported
		for face in mesh.faces:
			if face.image:
				if not self.matDict.has_key(face.image.name):
					print 'Image "%s" has no material defined, and cannot be exported!' % face.image.name
					print 'I will now exit'
					self.pipe.close()
					sys.exit()	

	def writeFaces(self):
		mesh=self.obj.getData()
		for face in mesh.faces:
			self.pipe.write('\tface\n')
			verts=[]
			norms=[]
			for v in face.v:
				data='%s %s %s' %(rnd(v[0],self.ro),rnd(v[1],self.ro),rnd(v[2],self.ro))
				truevec=v.no
				if not (mesh.mode & Blender.NMesh.Modes.NOVNORMALSFLIP):
					norm='%s %s %s' %(rnd(truevec[0],self.ro),rnd(truevec[1],self.ro),rnd(truevec[2],self.ro))
					norms.append(self.normalIndex.index(norm))
				verts.append(self.vertIndex.index(data))
			vstr=self.cleanStr(str(verts))
			if not (mesh.mode & Blender.NMesh.Modes.NOVNORMALSFLIP):nstr=self.cleanStr(str(norms))
			self.pipe.write('\t\tvertices %s\n' % vstr)
			if not (mesh.mode & Blender.NMesh.Modes.NOVNORMALSFLIP):self.pipe.write('\t\tnormals %s\n' %nstr)
			self.pipe.flush()
			if mesh.hasFaceUV() and face.uv and face.image:
				cords=[]
				for coord in face.uv:
					data='%s %s' %(rnd(coord[0],self.ro),rnd(coord[1],self.ro))
					cords.append(self.coordIndex.index(data))
				cstr=self.cleanStr(str(cords))
				self.pipe.write("\t\ttexcoords %s\n" %cstr)
				self.pipe.flush()
			if face.image:
				data=self.matDict[face.image.name]
			elif mesh.materials:
				data=mesh.materials[face.mat].name
			else:
				data=None
			if data:
				self.pipe.write('\t\tmatref %s\n' %data)
				self.pipe.flush()
			#set face bools from face.transp settings
			if face.transp:
				if 1&face.transp:
					self.pipe.write('\t\tshootthrough\n')
				if 2&face.transp:
					self.pipe.write('\t\tdrivethrough\n')				
				if 3&face.transp:
					self.pipe.write('\t\tpassable\n')				
				if 4&face.transp:
					self.pipe.write('\t\tsmoothbounce\n')
				if self.out.phydrv:
					data=self.out.phydrv
					self.pipe.write('\t\tphydrv %s\n' % data)
			self.pipe.write('\tendface\n')
			self.pipe.flush()

	def writeVertices(self):
		mesh=self.obj.getData()
		verts=mesh.verts
		for vert in verts:
			data='%s %s %s' %(rnd(vert[0],self.ro),rnd(vert[1],self.ro),rnd(vert[2],self.ro))
			self.vertIndex.append(data)
			self.pipe.write('\tvertex %s\n' %data)
			
	def writeInside(self):
		try:
			mesh=Blender.Object.Get('%s_I' % self.obj.name).getData()
		except ValueError:
			return
		verts=mesh.verts
		for vert in verts:
			data='%s %s %s' %(rnd(vert[0],self.ro),rnd(vert[1],self.ro),rnd(vert[2],self.ro))
			self.pipe.write('\tinside %s\n' %data)
		
	def writeOutside(self):
		try:
			mesh=Blender.Object.Get('%s_O' % self.obj.name).getData()
		except ValueError:
			return
		verts=mesh.verts
		for vert in verts:
			data='%s %s %s' %(rnd(vert[0],self.ro),rnd(vert[1],self.ro),rnd(vert[2],self.ro))
			self.pipe.write('\toutside %s\n' %data)	
	
	def writeLOD(self):
		try:
			text=Blender.Text.Get(obj.name)
		except NameError:
			return
		lines=text.asLines()
		for x in xrange(text.getNLines()+1):
			self.pipe.write("%s\n" % lines[x])
		self.pipe.flush()
	
	def makeMatDict(self):
		mats=Blender.Material.Get()
		for mat in mats:
			image=self.getImage(mat.name)
			if image:
				self.matDict[image.name]=mat.name

	def getImage(self,matref):
		mat=Blender.Material.Get(matref)
		try:
			tex=mat.getTextures()[0].tex
			img=tex.getImage()
		except AttributeError:
			#print 'Texture not found %s' % matref
			img=None
		return img
