# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****


import Blender
from Blender import NMesh, Image, Object
import BZcommon
from BZcommon import retList
from BZobject import bzmesh,BlObject
import BZtypeList


class importer(bzmesh):

	def __init__(self,bzobj):
		BZPrefs=Blender.Registry.GetKey('BZFlag', True)
		self.obj=bzobj
		self.matLib={}
		self.matLib['BZTexface']=0
		self.matIndex=0
		try:
			self.wallimage=Image.Get('mesh.png')
		except NameError:
			self.wallimage=Blender.Image.Load(BZPrefs['bztexpath']+'mesh.png')
			
	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(Blender.Material.Get('BZtexface'))
		if self.obj.matref:
			obj.addMaterial(Blender.Material.Get(self.obj.matref))
			self.matIndex+=1
			self.matLib[self.obj.matref]=self.matIndex
			cusImage=self.getimage(self.obj.matref)
		obj.hasFaceUV(1)
		dx,dy,dz=self.obj.position
		for verts in self.obj.vertex:
			x,y,z=float(verts[0])+dx,float(verts[1])+dy,float(verts[2])+dz
			v=NMesh.Vert(x,y,z)
			obj.verts.append(v)
		if self.obj.normals:
			try:
				i=0
				for v in obj.verts:
					x=float(self.obj.normals[i][0])
					y=float(self.obj.normals[i][2])
					z=float(self.obj.normals[i][1])
					vec=Blender.Mathutils.Vector([x,y,z])
					v.no=vec
					i+=1
			except IndexError:
				pass
				
		faces=[(1,2,3),(1,3,4),(1,4,2),(2,3,4),(3,2,1),(4,3,1),(4,3,2),(2,4,1)]
		for face in faces:
			f=NMesh.Face()
			uvs=[]
			for verts in face:
				f.v.append(obj.verts[verts-1])
				try:
					uvs.append(self.obj.texcoords[verts])
				except IndexError:
					uvs=[(0.0,1.0),(0.0,0.0),(1.0,0.0)]
			f.flag=1
			f.mode=5
			if self.obj.matref:
				f.mat=1
				if cusImage:f.image=cusImage
			else:
				f.mat=0
				f.image=self.wallimage
			f.uv=uvs
			obj.faces.append(f)
		
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.GetCurrent ()
		scene.link (scobj)
		self.updateto()	
		Blender.Redraw()
	#enddef

	def getimage(self,matref):
		mat=Blender.Material.Get(matref)
		try:
			tex=mat.getTextures()[0].tex
			img=tex.getImage()
		except AttributeError:
			#print 'Texture not found %s' % matref
			img=None
		return img
	#enddef

class export:
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.tetra()
		self.obj=blobj
		
	def run(self):
		pass
		
