# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import Material, NMesh, Object
import BZcommon
from BZobject import bzprim,BlObject
import BZtypeList


class importer(bzprim):

	def __init__(self,bzobj):
		try:
			material=Material.Get('BZwatermat')
		except NameError:
			material=Material.New('BZwatermat')
			material.rgbCol = [ 0.0, 0.2, 1.0 ]
			material.setAlpha(0.15)
			material.mode = Material.Modes.ZTRANSP
		self.sidemat=material
		self.obj=bzobj

	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(self.sidemat)
		for verts in self.obj.verts:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		f=NMesh.Face()
		for verts in [0,1,2,3]:
			f.v.append(obj.verts[verts])
			f.mat=0
			f.mode=4
		obj.faces.append(f)
		scobj=Blender.Object.New('Mesh',self.obj.name)
		scobj.setDrawType(3)
		scobj.setDrawMode(128)
		scobj.link(obj)
		scene = Blender.Scene.GetCurrent ()
		scene.link (scobj)
		self.updateto()
		Blender.Redraw()
	
	def updateto(self):
		BZcommon.setProperties(self.obj)
		obj=Blender.Object.Get(self.obj.name)
		obj.setLocation(0,0,float(self.obj.height[0]))
		obj.setSize(400,400,0)

class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.waterLevel()
		self.obj=blobj
	
	def run(self):
		self.updateFrom()
		self.pipe.write('waterLevel\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.pipe.write('\theight %s\n' % self.obj.getLocation()[2])
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
		
