# Copyright (C) 2006: A.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
import math
from matTemp import gun
from Blender import Object,NMesh
import BZcommon
from BZobject import BlObject
import BZtypeList

class importer:
	def __init__(self,bzobj):
		self.obj=bzobj
	
	def run(self):
		obj=NMesh.New(self.obj.name)
		for verts in gun.vertlist:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		for face in gun.facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			obj.faces.append(f)
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.GetCurrent ()
		scene.link (scobj)
		BZcommon.setProperties(self.obj)
		#scobj.layers=[2]
		if self.obj.tilt:
			x=BZcommon.deg2rad(self.obj.tilt[0])
		else:
			x=0
		z=BZcommon.deg2rad(self.obj.rotation[0])
		scobj.setEuler((math.pi/2,-x,-z))
		scobj.setLocation(float(self.obj.position[0]),float(self.obj.position[1]),float(self.obj.position[2]))			
		Blender.Redraw()
	
class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.weapon()
		self.obj=blobj
	
	def run(self):
		self.updateFrom()
		x,y,z=self.obj.getEuler()
		if z:
			self.out.rotation=BZcommon.rad2deg(-z)	
		if y:
			self.out.tilt=BZcommon.rad2deg(-y)
		self.pipe.write('weapon\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writePrim()
		self.pipe.write('end\n\n')
		self.pipe.flush()

	def writeStrings(self):
		for attrib in self.out.strings:
			data=getattr(self.out,attrib,None)
			if attrib=='name':
				pass
			elif data:
				if attrib=='ttype':attrib='type'
				self.pipe.write('\t%s %s\n' %(attrib,data))
		self.pipe.flush()
