# Copyright (C) 2006: A.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import Material, Texture, NMesh, Image, Object
import BZcommon
from BZobject import bzprim,BlObject
import BZtypeList


class importer(bzprim):

	def __init__(self,bzobj):
		try:
			material=Material.Get('BZzonemat')
		except NameError:
			material=Material.New('BZzonemat')
			material.rgbCol = [ 1.0, 1.0, 0.0 ]
			material.setAlpha(0.1)
			material.mode = Material.Modes.ZTRANSP
		self.sidemat=material
		self.obj=bzobj
	
	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(self.sidemat)
		for verts in self.obj.verts:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		for face in self.obj.faces:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
				f.mat=0
				f.mode=4
			obj.faces.append(f)
		scobj=Blender.Object.New('Mesh',self.obj.name)
		scobj.setDrawType(3)
		scobj.setDrawMode(128)
		scobj.link(obj)
		scene = Blender.Scene.GetCurrent ()
		scene.link (scobj)
		self.updateto()
		Blender.Redraw()
	
class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.zone()
		self.obj=blobj
	
	def run(self):
		BZPrefs=Blender.Registry.GetKey('BZFlag', True)
		ro=BZPrefs['ro']
		self.updateFrom()
		self.pipe.write('zone\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writePrim()
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
		

